from autohandshake.src import (
    # main classes
    HandshakeSession,
    HandshakeBrowser,
    UserType,
    ViewAsStudent,

    # pages
    AccessRequestPage,
    AppointmentCalendarPage,
    InsightsPage,
    AppointmentTypesListPage,
    AppointmentTypePage,
    MajorSettingsPage,
    FileType,
    InterviewSchedulePage,
    CareerInterestsPage,
    SurveyPage,
    WaitingRoomPage,
    RequestStatus,
    EventsPage,
    LabelSettingsPage,
    StaffPage
)

from autohandshake.__about__ import (
    __author__,
    __copyright__,
    __email__,
    __version__
)


name = "autohandshake"
