from autohandshake.src.Pages.SchoolSettings.AppointmentTypesListPage import AppointmentTypesListPage
from autohandshake.src.Pages.SchoolSettings.AppointmentTypePage import AppointmentTypePage
from autohandshake.src.Pages.SchoolSettings.MajorSettingsPage import MajorSettingsPage
from autohandshake.src.Pages.SchoolSettings.LabelSettingsPage import LabelSettingsPage
from autohandshake.src.Pages.SchoolSettings.StaffPage import StaffPage
